-- -------------------------------------------------------------------------- --
-- EnhancedTradeSkillUI by kunda                                              --
-- -------------------------------------------------------------------------- --
--                                                                            --
-- This a small simple addon to show more recipies in the profession window.  --
--                                                                            --
-- Four simple features:                                                      --
--  - Independent Frame, Move, Scale and Resize.                              --
--                                                                            --
-- Left-Click: (on Portrait TradeSkill Texture)                               --
--  - Move TradeSkill Frame.                                                  --
-- Right-Click: (on Portrait TradeSkill Texture)                              --
--  - Options (Resize and Scale)                                              --
--                                                                            --
-- -------------------------------------------------------------------------- --

EnhancedTradeSkillUI = CreateFrame("Frame") -- container
EnhancedTradeSkillUI_Options = {}           -- SavedVariable options table

local _, prg = ...
local L if not prg.L then return end for k, v in pairs(prg.L) do if type(v) ~= "string" then prg.L[k] = tostring(k) end end
      L = prg.L                             -- localization table
local maxTradeSkills = 50                   -- maximum number of shown TradeSkills
local minScale       = 0.5                  -- minimum value for TradeSkillFrame scale
local maxScale       = 1.5                  -- maximum value for TradeSkillFrame scale
local IsTradeSkillUI = false                -- check value for Blizzard_TradeSkillUI

-- ---------------------------------------------------------------------------------------------------------------------
local function SlashHandler()
	if InterfaceOptionsFrame and InterfaceOptionsFrame:IsShown() then
		InterfaceOptionsFrame_Show()
		HideUIPanel(GameMenuFrame)
	else
		InterfaceOptionsFrame_Show()
		InterfaceOptionsFrame_OpenToCategory("EnhancedTradeSkillUI")
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function EnhancedTradeSkillUI:InitOptions()
	SlashCmdList["ENHANCEDTRADESKILLUI"] = SlashHandler
	SLASH_ENHANCEDTRADESKILLUI1 = "/etsui"
	SLASH_ENHANCEDTRADESKILLUI2 = "/tradeskillui"
	SLASH_ENHANCEDTRADESKILLUI3 = "/enhancedtradeskillui"

	if EnhancedTradeSkillUI_Options.ShownSkills == nil then EnhancedTradeSkillUI_Options.ShownSkills = 8 end
	if EnhancedTradeSkillUI_Options.scale       == nil then EnhancedTradeSkillUI_Options.scale       = 1 end
	if EnhancedTradeSkillUI_Options.lock        == nil then EnhancedTradeSkillUI_Options.lock        = false end
	if EnhancedTradeSkillUI_Options.tooltip     == nil then EnhancedTradeSkillUI_Options.tooltip     = true end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function EnhancedTradeSkillUI:CreateInterfaceOptions()
	local options = CreateFrame("FRAME", "EnhancedTradeSkillUI_InterfaceOptions")
	options.name = "EnhancedTradeSkillUI"

	-- title & description START
	local title = options:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	title:SetPoint("TOPLEFT", 16, -16)
	title:SetText("EnhancedTradeSkillUI")
	title:SetJustifyH("LEFT")
	title:SetJustifyV("TOP")

	local subText = options:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
	subText:SetPoint("TOPLEFT", title, "BOTTOMLEFT", 0, -8)
	subText:SetText(L["Shows more recipies in the profession window."])
	subText:SetNonSpaceWrap(true)
	subText:SetJustifyH("LEFT")
	subText:SetJustifyV("TOP")
	-- title & description END

	-- shownSkills option START
	local shownSkills = CreateFrame("Slider", "EnhancedTradeSkillUI_InterfaceOptions_shownSkills", EnhancedTradeSkillUI_InterfaceOptions, "OptionsSliderTemplate")
	EnhancedTradeSkillUI_InterfaceOptions_shownSkills:SetWidth(200)
	EnhancedTradeSkillUI_InterfaceOptions_shownSkills:SetHeight(16)
	EnhancedTradeSkillUI_InterfaceOptions_shownSkills:SetPoint("TOPLEFT", subText, "BOTTOMLEFT", 8, -30)
	EnhancedTradeSkillUI_InterfaceOptions_shownSkillsText:SetPoint("LEFT")
	EnhancedTradeSkillUI_InterfaceOptions_shownSkillsText:SetText(L["Shown Skills"]..": "..EnhancedTradeSkillUI_Options.ShownSkills)
	EnhancedTradeSkillUI_InterfaceOptions_shownSkillsLow:SetText("8")
	EnhancedTradeSkillUI_InterfaceOptions_shownSkillsHigh:SetText(tostring(maxTradeSkills))
	EnhancedTradeSkillUI_InterfaceOptions_shownSkills:SetMinMaxValues(8, maxTradeSkills)
	EnhancedTradeSkillUI_InterfaceOptions_shownSkills:SetValueStep(1)
	EnhancedTradeSkillUI_InterfaceOptions_shownSkills:SetValue(EnhancedTradeSkillUI_Options.ShownSkills)
	EnhancedTradeSkillUI_InterfaceOptions_shownSkills:SetScript("OnValueChanged", function(self, value)
		value = math.floor(value+0.5)
		EnhancedTradeSkillUI_InterfaceOptions_shownSkillsText:SetText(L["Shown Skills"]..": "..value)
		EnhancedTradeSkillUI_Options.ShownSkills = value
		EnhancedTradeSkillUI:Resize(value, EnhancedTradeSkillUI_Options.scale)
	end)
	BlizzardOptionsPanel_Slider_Enable(EnhancedTradeSkillUI_InterfaceOptions_shownSkills)

	local shownSkillsText = options:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
	shownSkillsText:SetPoint("TOPLEFT", shownSkills, "BOTTOMLEFT", 0, -10)
	shownSkillsText:SetText("("..L["WoW Default"].." = 8)")
	shownSkillsText:SetTextColor(0.6, 0.6, 0.6, 1)
	shownSkillsText:SetNonSpaceWrap(true)
	-- shownSkills option END

	-- scale option START
	local scale = CreateFrame("Slider", "EnhancedTradeSkillUI_InterfaceOptions_scale", EnhancedTradeSkillUI_InterfaceOptions, "OptionsSliderTemplate")
	EnhancedTradeSkillUI_InterfaceOptions_scale:SetWidth(200)
	EnhancedTradeSkillUI_InterfaceOptions_scale:SetHeight(16)
	EnhancedTradeSkillUI_InterfaceOptions_scale:SetPoint("TOPLEFT", shownSkillsText, "BOTTOMLEFT", 0, -30)
	EnhancedTradeSkillUI_InterfaceOptions_scaleText:SetPoint("LEFT")
	EnhancedTradeSkillUI_InterfaceOptions_scaleText:SetText(L["Scale"]..": "..(EnhancedTradeSkillUI_Options.scale*100).."%")
	EnhancedTradeSkillUI_InterfaceOptions_scaleLow:SetText(tostring(minScale*100).."%")
	EnhancedTradeSkillUI_InterfaceOptions_scaleHigh:SetText(tostring(maxScale*100).."%")
	EnhancedTradeSkillUI_InterfaceOptions_scale:SetMinMaxValues(minScale, maxScale)
	EnhancedTradeSkillUI_InterfaceOptions_scale:SetValueStep(0.05)
	EnhancedTradeSkillUI_InterfaceOptions_scale:SetValue(EnhancedTradeSkillUI_Options.scale)
	EnhancedTradeSkillUI_InterfaceOptions_scale:SetScript("OnValueChanged", function(self, value)
		value = ceil(value*100-0.5) / 100
		EnhancedTradeSkillUI_InterfaceOptions_scaleText:SetText(L["Scale"]..": "..(value*100).."%")
		EnhancedTradeSkillUI_Options.scale = value
		EnhancedTradeSkillUI:Resize(EnhancedTradeSkillUI_Options.ShownSkills, value)
	end)
	BlizzardOptionsPanel_Slider_Enable(EnhancedTradeSkillUI_InterfaceOptions_scale)

	local scaleText = options:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
	scaleText:SetPoint("TOPLEFT", scale, "BOTTOMLEFT", 0, -10)
	scaleText:SetText("("..L["WoW Default"].." = 100%)")
	scaleText:SetTextColor(0.6, 0.6, 0.6, 1)
	scaleText:SetNonSpaceWrap(true)
	-- scale option END

	-- lock frame pos START
	local lockFrame = CreateFrame("CheckButton", "EnhancedTradeSkillUI_InterfaceOptions_lockFrame", EnhancedTradeSkillUI_InterfaceOptions, "InterfaceOptionsCheckButtonTemplate")
	EnhancedTradeSkillUI_InterfaceOptions_lockFrame:SetPoint("TOPLEFT", scaleText, "TOPLEFT", -8, -20)
	EnhancedTradeSkillUI_InterfaceOptions_lockFrameText:SetText(L["Lock"])
	EnhancedTradeSkillUI_InterfaceOptions_lockFrame:SetChecked(EnhancedTradeSkillUI_Options.lock)
	EnhancedTradeSkillUI_InterfaceOptions_lockFrame:SetScript("OnClick", function(self)
		EnhancedTradeSkillUI_Options.lock = not EnhancedTradeSkillUI_Options.lock
	end)
	lockFrame.tooltipText = L["Locks or unlocks the TradeSkillFrame for movement."]
	-- lock frame pos END

	-- portait tooltip START
	local PortraitTooltip = CreateFrame("CheckButton", "EnhancedTradeSkillUI_InterfaceOptions_PortraitTooltip", EnhancedTradeSkillUI_InterfaceOptions, "InterfaceOptionsCheckButtonTemplate")
	EnhancedTradeSkillUI_InterfaceOptions_PortraitTooltip:SetPoint("TOPLEFT", lockFrame, "TOPLEFT", 0, -20)
	EnhancedTradeSkillUI_InterfaceOptions_PortraitTooltipText:SetText(L["Portrait Tooltip"])
	EnhancedTradeSkillUI_InterfaceOptions_PortraitTooltip:SetChecked(EnhancedTradeSkillUI_Options.tooltip)
	EnhancedTradeSkillUI_InterfaceOptions_PortraitTooltip:SetScript("OnClick", function(self)
		EnhancedTradeSkillUI_Options.tooltip = not EnhancedTradeSkillUI_Options.tooltip
	end)
	-- portait tooltip END

	InterfaceOptions_AddCategory(options)
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function EnhancedTradeSkillUI:Init()
	EnhancedTradeSkillUI:SetUIPanel(TradeSkillFrame, nil)
	table.insert(UISpecialFrames,"TradeSkillFrame")

	TradeSkillFrame:SetClampedToScreen(true)

	local button = CreateFrame("Button", "EnhancedTradeSkillUI_Mover", TradeSkillFrame)
	button:ClearAllPoints()
	button:SetPoint("CENTER", "TradeSkillFramePortrait", "CENTER", 0, 0)
	button:SetFrameLevel(TradeSkillFrame:GetFrameLevel()+1)
	button:SetWidth(55)
	button:SetHeight(55)
	button:RegisterForClicks("LeftButtonDown", "RightButtonUp")
	button:SetScript("OnEnter", function(self, button)
		if EnhancedTradeSkillUI_Options.tooltip then
			GameTooltip:SetOwner(self, "ANCHOR_BOTTOMRIGHT", 0, 50)
			if EnhancedTradeSkillUI_Options.lock then
				GameTooltip:SetText(L["Right-Click: Options"], nil, nil, nil, nil, 1)
			else
				GameTooltip:SetText(L["Left-Click: Move"].."\n"..L["Right-Click: Options"], nil, nil, nil, nil, 1)
			end
		end
	end)
	button:SetScript("OnLeave", function(self, button)
		GameTooltip:Hide()
	end)
	button:SetScript("OnMouseDown", function(self, button)
		GameTooltip:Hide()
		if button == "RightButton" then
			SlashHandler()
		else
			if not EnhancedTradeSkillUI_Options.lock then
				TradeSkillFrame:StartMoving()
			end
		end
	end)
	button:SetScript("OnMouseUp", function(self, button)
		if button == "RightButton" then return end
		if not EnhancedTradeSkillUI_Options.lock then
			TradeSkillFrame:StopMovingOrSizing()
			EnhancedTradeSkillUI:SaveFramePos()
		end
	end)
	button:SetScript("OnDragStop", function(self, button)
		if button == "RightButton" then return end
		if not EnhancedTradeSkillUI_Options.lock then
			TradeSkillFrame:StopMovingOrSizing()
		end
	end)

	if EnhancedTradeSkillUI_Options.posX then
		TradeSkillFrame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", EnhancedTradeSkillUI_Options.posX, EnhancedTradeSkillUI_Options.posY)
	end
end

function EnhancedTradeSkillUI:SetUIPanel(frame, info)
	local function NOOP() end
	local name = frame:GetName()
	local visible = frame:IsShown()
	local script1 = frame:GetScript("OnShow")
	local script2 = frame:GetScript("OnHide")

	if visible then
		frame:HookScript("OnShow", NOOP)
		frame:HookScript("OnHide", NOOP)
		HideUIPanel(frame)
	end

	UIPanelWindows[name] = info

	if visible then
		ShowUIPanel(frame)
		frame:HookScript("OnShow", script1)
		frame:HookScript("OnHide", script2)
	end
end

function EnhancedTradeSkillUI:SaveFramePos()
	EnhancedTradeSkillUI_Options.posX = TradeSkillFrame:GetLeft()
	EnhancedTradeSkillUI_Options.posY = TradeSkillFrame:GetTop()
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function EnhancedTradeSkillUI:CreateNewTextButtons()
	for i=9, maxTradeSkills do
		local b = CreateFrame("Button", "TradeSkillSkill"..i, TradeSkillFrame, "TradeSkillSkillButtonTemplate")
		b:SetPoint("TOPLEFT", "TradeSkillSkill"..(i-1), "BOTTOMLEFT", 0, 0)
	end

	local scrollbarTexture = TradeSkillListScrollFrame:CreateTexture("EnhancedTradeSkillUI_ScrollbarTexture", "BACKGROUND")
	scrollbarTexture:ClearAllPoints()
	scrollbarTexture:SetPoint("TOPRIGHT", TradeSkillListScrollFrame, "TOPRIGHT", 29, -50)
	scrollbarTexture:SetWidth(32)
	scrollbarTexture:SetHeight(0)
	scrollbarTexture:SetTexture("Interface\\ClassTrainerFrame\\UI-ClassTrainer-ScrollBar")
	scrollbarTexture:SetTexCoord(0/64, 32/64, 25/128, 123/128)
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function EnhancedTradeSkillUI:Resize(newsize, scale)
	if not IsTradeSkillUI then return end

	TRADE_SKILLS_DISPLAYED = newsize

	TradeSkillFrame:SetHeight(296+newsize*16)

	TradeSkillListScrollFrame:SetHeight(2+newsize*16)
	TradeSkillListScrollFrameScrollChildFrame:SetHeight(16*newsize-60)

	if newsize > 14 then
		EnhancedTradeSkillUI_ScrollbarTexture:Show()
		EnhancedTradeSkillUI_ScrollbarTexture:SetHeight((16*newsize)-(16*8))
	else
		EnhancedTradeSkillUI_ScrollbarTexture:Hide()
		EnhancedTradeSkillUI_ScrollbarTexture:SetHeight(0)
	end

	TradeSkillFrame:SetClampRectInsets(0, 0, 0, 210+(newsize*16))

	TradeSkillHorizontalBarLeft:SetPoint("TOPLEFT", "TradeSkillFrame", "TOPLEFT", 0, -207-(newsize-8)*16)
	TradeSkillDetailScrollFrame:SetPoint("TOPLEFT", "TradeSkillFrame", "TOPLEFT", 6, -220-(newsize-8)*16)

	for i = newsize+1, maxTradeSkills do
		_G["TradeSkillSkill"..i]:Hide()
	end

	TradeSkillFrame:SetScale(scale)
	TradeSkillFrame_Update()
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
local function OnEvent(self, event, arg1)
	if event == "ADDON_LOADED" then
		if arg1 == "EnhancedTradeSkillUI" then
			EnhancedTradeSkillUI:InitOptions()
			EnhancedTradeSkillUI:CreateInterfaceOptions()
		elseif arg1 == "Blizzard_TradeSkillUI" then
			if IsTradeSkillUI then return end
			IsTradeSkillUI = true
			EnhancedTradeSkillUI:Init()
			EnhancedTradeSkillUI:CreateNewTextButtons()
			EnhancedTradeSkillUI:Resize(EnhancedTradeSkillUI_Options.ShownSkills, EnhancedTradeSkillUI_Options.scale)
		end
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

EnhancedTradeSkillUI:RegisterEvent("ADDON_LOADED")
EnhancedTradeSkillUI:SetScript("OnEvent", OnEvent)